/*
 * GPScan - Graphical Port Scanner
 * Copyright (C) 2001-2003, split_yo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include "about.h"

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define PACKAGE_AUTHOR	"Copyright (C) 2001-2009 Angel Leon"
#define PACKAGE_DESC	"Graphical Port Scanner"

#ifndef PACKAGE_STRING
# define PACKAGE_STRING	"GPScan"
#endif

#define WINT		"Aceca de ..."
#define WINB		5
#define WINW		300
#define WINH		250
#define N_LAYOUTS	3

static GtkWidget*		_GpsAboutWindow = NULL;
static GdkGC*			_GpsAboutGC[N_LAYOUTS];
static GdkColor			_GpsAboutColor[N_LAYOUTS];
static PangoLayout*		_GpsAboutLayout[N_LAYOUTS];
static PangoFontDescription*	_GpsAboutFont[N_LAYOUTS];

static gboolean
gps_about_draw_expose(GtkWidget* wi, GdkEventExpose* e, gpointer data) {
	gint w[N_LAYOUTS], h[N_LAYOUTS];
	gint x[N_LAYOUTS], y[N_LAYOUTS];
	gint i, dw=0, dh=0;

	gdk_window_get_geometry(wi->window, NULL, NULL, &dw, &dh, NULL);

	for(i=0; i < N_LAYOUTS; i++) {
		w[i] = h[i] = x[i] = y[i] = 0;
		pango_layout_get_pixel_size(_GpsAboutLayout[i], &w[i], &h[i]);
	}

	x[0] = (dw - w[0]) / 2;
	y[0] = WINB;

	x[1] = (dw - w[1]) / 2;
	y[1] = (dh - h[1] + WINB) / 2;

	x[2] = (dw - w[2]) / 2;
	y[2] = dh - WINB - h[2];

	for(i=0; i < N_LAYOUTS; i++) {
		gdk_draw_layout(wi->window, _GpsAboutGC[i], x[i], y[i],
				_GpsAboutLayout[i]);
	}

	return TRUE;
}

static gboolean
gps_about_draw_configure(GtkWidget* wi, GdkEventConfigure* e, gpointer d) {
	if (_GpsAboutLayout[0]) {
		gint i, w=0, h=0;

		gdk_window_get_geometry(wi->window, NULL, NULL, &w, &h, NULL);
		w -= (WINB * 2);

		for(i=0; i < N_LAYOUTS; i++) {
			pango_layout_set_width(_GpsAboutLayout[i],
					w * PANGO_SCALE);
		}
	}

	return TRUE;
}

static void
gps_about_set_color(GdkColor* c, guint16 r, guint16 g, guint16 b) {
	if (!c)
		return;
	c->red = r;
	c->green = g;
	c->blue = b;
}

static void
gps_about_draw_realize(GtkWidget* wi, gpointer data) {
	PangoContext* context = NULL;
	GdkColormap* cmap = NULL;
	GdkColor c;
	gint i;
	gchar* text_str[] = {
		PACKAGE_STRING, PACKAGE_DESC, PACKAGE_AUTHOR
	};

	/* Obtenemos los recursos a usar */
	context = gtk_widget_get_pango_context(wi);
	cmap = gtk_widget_get_colormap(wi);

	gps_about_set_color(&c, 65535, 65535, 65535);
	gdk_colormap_alloc_color(cmap, &c, TRUE, TRUE);
	gtk_widget_modify_bg(wi, GTK_STATE_NORMAL, &c);

	/* Creamos los recursos usados para los mensajes */
	for(i=0; i < N_LAYOUTS; i++) {
		_GpsAboutGC[i] = NULL;
		_GpsAboutFont[i] = NULL;
		_GpsAboutLayout[i] = NULL;

		_GpsAboutGC[i] = gdk_gc_new(wi->window);
		_GpsAboutFont[i] = pango_font_description_new();
		_GpsAboutLayout[i] = pango_layout_new(context);
	}

	gps_about_set_color(&_GpsAboutColor[0], 65535, 0, 0);
	pango_font_description_set_size(_GpsAboutFont[0], 20 * PANGO_SCALE);
	pango_font_description_set_weight(_GpsAboutFont[0], PANGO_WEIGHT_BOLD);

	gps_about_set_color(&_GpsAboutColor[1], 32767, 32767, 32767);
	pango_font_description_set_size(_GpsAboutFont[1], 14 * PANGO_SCALE);

	gps_about_set_color(&_GpsAboutColor[2], 0, 0, 49149);
	pango_font_description_set_size(_GpsAboutFont[2], 10 * PANGO_SCALE);

	for(i=0; i < N_LAYOUTS; i++) {
		gdk_gc_set_rgb_fg_color(_GpsAboutGC[i], &_GpsAboutColor[i]);
		pango_layout_set_text(_GpsAboutLayout[i], text_str[i], -1);
		pango_layout_set_font_description(_GpsAboutLayout[i],
				_GpsAboutFont[i]);
	}
}

static void
gps_about_window_destroy(GtkWidget *w, gpointer data) {
	if (_GpsAboutWindow) {
		gint i;

		gtk_widget_destroy(_GpsAboutWindow);
		_GpsAboutWindow = NULL;

		for(i=0; i < N_LAYOUTS; i++) {
			pango_font_description_free(_GpsAboutFont[i]);
			g_object_unref(_GpsAboutLayout[i]);
			g_object_unref(_GpsAboutGC[i]);

			_GpsAboutLayout[i] = NULL;
			_GpsAboutFont[i] = NULL;
			_GpsAboutGC[i] = NULL;
		}
	}
}

static void
gps_about_window_show(void) {
	GtkWidget* frame;
	GtkWidget* draw;
	GtkWidget* button;

	if (_GpsAboutWindow) {
		gdk_window_raise(_GpsAboutWindow->window);
		return;
	}

	_GpsAboutWindow = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(_GpsAboutWindow), WINT);
	gtk_window_set_position(GTK_WINDOW(_GpsAboutWindow),
			GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(_GpsAboutWindow),
			WINW, WINH);
	g_signal_connect(G_OBJECT(_GpsAboutWindow), "delete_event",
			 G_CALLBACK(gps_about_window_destroy), NULL);

	frame = gtk_frame_new(NULL);
	gtk_container_set_border_width(GTK_CONTAINER(frame), WINB);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(_GpsAboutWindow)->vbox),
			frame, TRUE, TRUE, 0);

	draw = gtk_drawing_area_new();
	gtk_container_add(GTK_CONTAINER(frame), draw);
	g_signal_connect(G_OBJECT(draw), "realize",
			G_CALLBACK(gps_about_draw_realize), NULL);
	g_signal_connect(G_OBJECT(draw), "configure_event",
			G_CALLBACK(gps_about_draw_configure), NULL);
	g_signal_connect(G_OBJECT(draw), "expose_event",
			G_CALLBACK(gps_about_draw_expose), NULL);
	gtk_widget_show(draw);
	gtk_widget_show(frame);

	button = gtk_button_new_with_label("Cerrar");
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_about_window_destroy), NULL);
	gtk_box_pack_end(GTK_BOX(GTK_DIALOG(_GpsAboutWindow)->action_area),
			button, FALSE, FALSE, 0);
	gtk_widget_grab_focus(button);
	gtk_widget_show(button);

	gtk_widget_show(_GpsAboutWindow);
}

static void
gps_about_cmdline_show(FILE* fd) {
	g_fprintf(fd, "%s\n%s\n%s\n", 
		PACKAGE_STRING, PACKAGE_AUTHOR, PACKAGE_DESC);
}

extern GtkWidget*	_GpsMainWindow;

void
gps_about_show(void) {
	if (!_GpsMainWindow)
		gps_about_cmdline_show(stdout);
	else
		gps_about_window_show();
}
