/*
 * GPScan - Graphical Port Scanner
 * Copyright (C) 2001-2003,  split_yo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <pthread.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <gtk/gtk.h>
#include "connect.h"
#include "window.h"

static pthread_t	_GpsScanThread;
static gboolean		_GpsScanStatus = FALSE;

void
gps_scan_stop(GPHost* h) {
	if (!_GpsScanStatus) {
		print_error("No se ha encontrado un proceso a detener\n");
		return;
	}

	if (pthread_cancel(_GpsScanThread) != 0) {
		print_func_error("scan_stop", "pthread_cancel");
		return;
	}
	_GpsScanStatus = FALSE;

	print_info("El proceso ha sido detenido por el usuario\n");
}

static void*
scan_exec(void *data) {
	GPHost* h = (GPHost*)data;
	gchar buf[512];
	gint sp=0, ep=0, f=0;
	gint i, j;

	sp = gps_host_get_port_start(h);
	ep = gps_host_get_port_end(h);
	f = gps_host_get_family(h);

	for (i=sp, j=0; i <= ep; i++) {
		struct sockaddr_in dir;
		gint fd=0;

		memset(buf, 0, sizeof(buf));
		memset(&dir, 0, sizeof(dir));

		dir.sin_family = f;
		dir.sin_port = htons(i);

#if HAVE_INET_PTON
		if (inet_pton(f, gps_host_get_ip(h), &dir.sin_addr) <= 0) {
			print_func_error("scan_exec", "inet_pton");
			pthread_exit(NULL);
		}
#elif HAVE_INET_ATON
		if (!inet_aton(gps_host_get_ip(h), &dir.sin_addr)) {
			print_func_error("scan_exec", "inet_aton");
			pthread_exit(NULL);
		}
#endif

		if ((fd = socket(f, SOCK_STREAM, IPPROTO_IP)) < 0) {
			print_func_error("scan_exec", "socket");
			pthread_exit(NULL);
		}

		if (connect(fd, (struct sockaddr *)&dir, sizeof(dir)) == 0) {
			char serv[128];

			memset(serv, 0, sizeof(serv));
			j++;

/*
			if (server->options & OPT_IDENT_SERV) {
				struct servent *sebuf;

				if ((sebuf = getservbyport(htons(i), NULL))) {
					sprintf(serv, "( %s / %s ) ",
						sebuf->s_name, sebuf->s_proto);
				} else
					sprintf(serv, "( desconocido ) ");
			}
*/

			print_scan(i, serv);
		}

		if (close(fd) < 0) {
			print_func_error("scan_exec", "close");
			pthread_exit(NULL);
		}
	}

	_GpsScanStatus = FALSE;
	print_result(j);

	pthread_exit(NULL);
}

void
gps_scan_start(GPHost* h) {
	char buf[512];

	if (_GpsScanStatus) {
		print("Ya se esta realizando un escaneo\n");
		return;
	}

	if (!gps_host_resolve(h)) {
		print_error(gps_host_get_error(h));
		return;
	}

	if (pthread_create(&_GpsScanThread, NULL, scan_exec, (void*)h) != 0) {
		print_func_error("scan_init", "pthread_create");
		return;
	}
	_GpsScanStatus = TRUE;

	{
		char* n = gps_host_get_name(h);
		char* ip = gps_host_get_ip(h);

		print("\nEscaneando el servidor ");
		print_type("red", n);

		if (strcmp(ip, n) != 0) {
			sprintf(buf, " (%s)", ip);
			print_type("blue", buf);
		}
		print("\n");
	}
}
