/*
 * GPScan - Graphical Port Scanner
 * Copyright (C) 2001-2003, split_yo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include <netdb.h>
#include "host.h"
#include "util.h"

struct _GPHost {
	char* name;
	char* ip;
	int family;
	int port_init;
	int port_fini;
	char* error;
};

GPHost* gps_host_new(void) {
	return g_malloc0(sizeof(GPHost));
}

void gps_host_free(GPHost* h) {
	if (!h)
		return;
	gps_string_free(&h->error);
	gps_string_free(&h->ip);
	gps_string_free(&h->name);
	g_free(h);
}

void gps_host_set_name(GPHost* h, const char* n) {
	if (!h)
		return;
	gps_string_set(&h->name, n);
}

void gps_host_set_ip(GPHost* h, const char* ip) {
	if (!h)
		return;
	gps_string_set(&h->ip, ip);
}

void gps_host_set_family(GPHost* h, int f) {
	if (!h)
		return;
	h->family = f;
}

void gps_host_set_port_start(GPHost* h, int n) {
	if (!h)
		return;
	h->port_init = n;
}

void gps_host_set_port_end(GPHost* h, int n) {
	if (!h)
		return;
	h->port_fini = n;
}

void gps_host_set_error(GPHost* h, const char* e) {
	if (!h)
		return;
	gps_string_set(&h->error, e);
}

char* gps_host_get_name(const GPHost* h) {
	return (h) ? h->name : NULL;
}

char* gps_host_get_ip(const GPHost* h) {
	return (h) ? h->ip : NULL;
}

int gps_host_get_family(const GPHost* h) {
	return (h) ? h->family : -1;
}

int gps_host_get_port_start(const GPHost* h) {
	return (h) ? h->port_init : -1;
}

int gps_host_get_port_end(const GPHost* h) {
	return (h) ? h->port_fini : -1;
}

char* gps_host_get_error(const GPHost* h) {
	return (h) ? h->error : NULL;
}

int gps_host_resolve(GPHost* h) {
	struct hostent* sh = NULL;
	char* n = NULL;
	char ip[20] = "";

	if (!h || !(n = gps_host_get_name(h))) {
		gps_host_set_error(h, "No ha especificado el servidor");
		return FALSE;
	}

	if (!(sh = gethostbyname(n))) {
		gps_host_set_error(h, "No se pudo localizar el servidor");
		return FALSE;
	}

	g_sprintf(ip, "%d.%d.%d.%d",
		(unsigned char)sh->h_addr_list[0][0],
		(unsigned char)sh->h_addr_list[0][1],
		(unsigned char)sh->h_addr_list[0][2],
		(unsigned char)sh->h_addr_list[0][3]);
	gps_host_set_ip(h, ip);
	gps_host_set_family(h, sh->h_addrtype);

	return TRUE;
}
