/*
 * GPScan - Graphical Port Scanner
 * Copyright (C) 2001-2003,  split_yo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <stdlib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include "about.h"
#include "host.h"
#include "window.h"

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_GETOPT_H
# include <getopt.h>
#endif

int str_to_num(char *str) {
	char buf[10];
	int i;

	for (i=0; *str != 0; str++)
		if (isdigit(*str))
			buf[i++] = *str;
		else
			return 0;
	buf[i] = 0;

	return atoi(buf);
}

void show_basic_help(char *pn) {
	g_fprintf(stderr, "Usage: %s [options] [server]\n"
		"Try `%s --help' for more information\n", pn, pn);
}

void show_info(void) {
	g_fprintf(stdout, "%s\n"
		"Compile date '%s'\n"
		"System: %s\n", PACKAGE_STRING, COMPILE_DATE, COMPILE_SYS);
}

void show_options(char *pn) {
	g_fprintf(stdout,
		"Uso: %s [opciones] [servidor]\n"
		"Opciones:\n"
		"  -f, --finish <puerto>  Indica el puerto de termino\n"
		"  -h, --help             Muestra la lista de opciones\n"
		"  -i, --info             Muestra informacion del programa\n"
		"  -s, --start  <puerto>  Indica el puerto de inicio\n"
		"  -v, --version          Muestra la version del programa\n",
		pn);
}

#if HAVE_GETOPT_LONG
struct option _GpsCmdLineOpt[] = {
	{ "finish", TRUE, NULL, 'f' },
	{ "help", FALSE, NULL, 'h' },
	{ "info", FALSE, NULL, 'i' },
	{ "start", TRUE, NULL, 's' },
	{ "version", FALSE, NULL, 'v' },
	{ 0, 0, 0, 0 },
};
char *_GpsCmdLineStr = "f:his:v";
#endif

int main(int argc, char **argv)
{
	GPHost *h = NULL;
	char* host = NULL;
	gint port_start = 0;
	gint port_end = 0;
#if HAVE_GETOPT_LONG
	char ch;
#endif

#if HAVE_GETOPT_LONG
	opterr = 0;
	while (TRUE) {
		ch = getopt_long(argc, argv, _GpsCmdLineStr, _GpsCmdLineOpt,
				NULL);
		if (ch == -1)
			break;

		switch(ch) {
			case 'f':
				if (!(port_end = str_to_num(optarg))) {
					g_fprintf(stderr,
						"El puerto solo puede contener numeros\n");
					return -1;
				}
				break;
			case 'h':
				show_options(argv[0]);
				return 0;
			case 'i':
				show_info();
				return 0;
			case 's':
				if (!(port_start = str_to_num(optarg))) {
					g_fprintf(stderr,
						"El puerto solo puede contener numeros\n");
					return -1;
				}
				break;
			case 'v':
				gps_about_show();
				return 0;
			case '?':
				show_basic_help(argv[0]);
				return -1;
			case ':':
				printf("Falto un parametro\n");
				return -1;
			default:
				printf("Error: main -> getopt_long\n");
				return -1;
		}
	}

	if (argc > optind)
		host = argv[optind];
#else
	if (argc >= 2)
		host = argv[1];
#endif

	if (!(h = gps_host_new())) {
		g_fprintf(stdout, "No se pudo crear la estructura basica\n");
		return -1;
	}
	gps_host_set_name(h, host);
	gps_host_set_port_start(h, port_start);
	gps_host_set_port_end(h, port_end);

	gtk_init(&argc, &argv);
	gps_main_window_show(h);
	gtk_main();

	gps_host_free(h);

	return 0;
}
