/*
 * GPScan - Graphical Port Scanner
 * Copyright (C) 2001-2003,  split_yo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <time.h>
#include <stdlib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include "about.h"
#include "connect.h"
#include "window.h"

#if HAVE_CONFIG_H
# include <config.h>
#endif

#define WINT	PACKAGE_STRING
#define WINB	5
#define WINW	500
#define WINH	300

#define PORT_MIN	1
#define PORT_MAX	65535
#define DEF_PORT_MIN	1
#define DEF_PORT_MAX	1024

GtkWidget*		_GpsMainWindow = NULL;	// Usado tambien por about.c
static GtkWidget*	_GpsMainText = NULL;
static GtkTextBuffer*	_GpsMainBuffer = NULL;
static GtkWidget*	entry_server = NULL;
static GtkWidget*	spin_sport = NULL;
static GtkWidget*	spin_eport = NULL;
static GtkWidget*	_GpsSaveWindow = NULL;

static void
gps_main_window_destroy(GtkWidget*, gpointer);

static void
gps_save_text_content(gchar *filename) {
	GtkTextIter siter, eiter;
	FILE *file;
	time_t t;
	gchar buf[512];
	gchar *tmp;

	gtk_text_buffer_get_start_iter(_GpsMainBuffer, &siter);
	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &eiter);

	tmp = gtk_text_buffer_get_text(_GpsMainBuffer, &siter, &eiter, TRUE);

	if (!(file = fopen(filename, "w"))) {
		print_func_error("save_text_content", "fopen");
		g_free(tmp);
		return;
	}

	t = time(NULL);
	g_sprintf(buf, "Fecha: %s", ctime(&t));
	fputs(buf, file);

	fputs(tmp, file);
	g_free(tmp);

	if (fclose(file) != 0) {
		print_func_error("save_text_content", "fclose");
		return;
	}
}

static void
gps_save_window_destroy(void) {
	if (_GpsSaveWindow) {
		gtk_widget_destroy(_GpsSaveWindow);
		_GpsSaveWindow = NULL;
	}
}

static void
gps_save_window_response(GtkDialog* dialog, gint response, gpointer data) {
	gchar* fn;

	switch(response) {
		case GTK_RESPONSE_NONE:
		case GTK_RESPONSE_DELETE_EVENT:
		case GTK_RESPONSE_CANCEL:
			gps_save_window_destroy();
			break;
		case GTK_RESPONSE_OK:
			fn = (gchar*)gtk_file_selection_get_filename(
					GTK_FILE_SELECTION(dialog));
			if (!fn || strlen(fn) <= 0) {
				print_error("No ha indicado el nombre del archivo\n");
				return;
			}
			gps_save_text_content(fn);
			gps_save_window_destroy();
			break;
	}
}

static void
gps_save_window_show(const GPHost* h) {
	char* n = NULL;
	gchar buf[256];

	if (_GpsSaveWindow) {
		gdk_window_raise(_GpsSaveWindow->window);
		return;
	}

	_GpsSaveWindow = gtk_file_selection_new("Guardar registro");
	g_signal_connect(G_OBJECT(_GpsSaveWindow), "response",
			G_CALLBACK(gps_save_window_response), NULL);
	if ((n = gps_host_get_name(h))) {
		sprintf(buf, "%s.log", n);
		gtk_file_selection_set_filename(
				GTK_FILE_SELECTION(_GpsSaveWindow), buf);
	}
	gtk_widget_show(_GpsSaveWindow);
}

void print(gchar *message) {
	GtkTextIter iter;

	if (!message || (strlen(message) <= 0))
		return;

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);
	gtk_text_buffer_insert(_GpsMainBuffer, &iter, message, strlen(message));
}

void print_type(gchar *type, gchar *message) {
	GtkTextIter iter;

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);

	gtk_text_buffer_insert_with_tags_by_name(_GpsMainBuffer, &iter,
				message, strlen(message), type, NULL);
}

void print_type_with_data(GtkTextBuffer *buffer, GtkTextIter *iter,
			  gchar *type, gchar *message) {
	gtk_text_buffer_insert_with_tags_by_name(buffer, iter,
				message, strlen(message), type, NULL);
}

void print_info(gchar *message) {
	GtkTextIter iter;

	if (!message || (strlen(message) <= 0))
		return;

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);
	gtk_text_buffer_insert_with_tags_by_name(_GpsMainBuffer, &iter,
		message, strlen(message), "blue", "bold", NULL);
}

void print_result(gint num) {
	GtkTextIter iter;
	gchar s1[56], s2[56];

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);

	if (!num) {
		sprintf(s1, "No se encontro algun puerto abierto\n");
		gtk_text_buffer_insert(_GpsMainBuffer, &iter, s1, strlen(s1));
		return;
	} else if (num == 1) {
		sprintf(s1, "Solo se encontro un puerto abierto\n");
		gtk_text_buffer_insert(_GpsMainBuffer, &iter, s1, strlen(s1));
		return;
	} else {
		sprintf(s1, "Se encontraron ");
		sprintf(s2, "puertos abiertos\n");
	}
	gtk_text_buffer_insert(_GpsMainBuffer, &iter, s1, strlen(s1));

	sprintf(s1, "%d ", num);
	print_type_with_data(_GpsMainBuffer, &iter, "red", s1);

	gtk_text_buffer_insert(_GpsMainBuffer, &iter, s2, strlen(s2));
}

void print_scan(gint port, gchar *service) {
	GtkTextIter iter;
	gchar buf[512];

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);

	sprintf(buf, "El puerto ");
	gtk_text_buffer_insert(_GpsMainBuffer, &iter, buf, strlen(buf));

	sprintf(buf, "%d ", port);
	print_type_with_data(_GpsMainBuffer, &iter, "red", buf);

	if (strlen(service) >= 1)
		print_type_with_data(_GpsMainBuffer, &iter, "blue", service);

	sprintf(buf, "esta abierto\n");
	gtk_text_buffer_insert(_GpsMainBuffer, &iter, buf, strlen(buf));
}

void print_func_error(gchar *func, gchar *sub) {
	GtkTextIter iter;
	gchar buf[512];

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);

	sprintf(buf, "Error: ");
	print_type_with_data(_GpsMainBuffer, &iter, "bold", buf);

	sprintf(buf, "%s ", func);
	print_type_with_data(_GpsMainBuffer, &iter, "red", buf);

	sprintf(buf, "-> ");
	gtk_text_buffer_insert(_GpsMainBuffer, &iter, buf, strlen(buf));

	sprintf(buf, "%s\n", sub);
	print_type_with_data(_GpsMainBuffer, &iter, "blue", buf);
}

void print_error(gchar *message) {
	GtkTextIter iter;

	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &iter);
	print_type_with_data(_GpsMainBuffer, &iter, "bold", "Error: ");
	print_type_with_data(_GpsMainBuffer, &iter, "red", message);
}

static void
gps_main_window_check_ident_toggled(GtkToggleButton *button, gpointer data) {
	GPHost* h = (GPHost*)data;
	gboolean status = gtk_toggle_button_get_active(button);
}

static void
gps_main_window_button_start_clicked(GtkWidget *w, gpointer data) {
	GPHost* h = (GPHost*)data;
	gchar* n = NULL;
	gint sp=0, ep=0;

	n = (gchar*)gtk_entry_get_text(GTK_ENTRY(entry_server));
	if (strlen(n) <= 0) {
		print_error("No ha indicado la direccion del servidor\n");
		gtk_widget_grab_focus(entry_server);
		return;
	}

	sp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_sport));
	ep = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_eport));

	gps_host_set_name(h, n);
	gps_host_set_port_start(h, sp);
	gps_host_set_port_end(h, ep);

	gps_scan_start(h);
}

static void
gps_main_window_button_stop_clicked(GtkWidget *w, gpointer data) {
	gps_scan_stop((GPHost*)data);
}

static void
gps_main_window_button_clear_clicked(GtkWidget* w, gpointer data) {
	GtkTextIter siter, eiter;

	gtk_text_buffer_get_start_iter(_GpsMainBuffer, &siter);
	gtk_text_buffer_get_end_iter(_GpsMainBuffer, &eiter);
	gtk_text_buffer_delete(_GpsMainBuffer, &siter, &eiter);
}

static void
gps_main_window_button_save_clicked(GtkWidget* w, gpointer data) {
	gps_save_window_show((GPHost*)data);
}

static void
gps_main_window_button_quit_clicked(GtkWidget* w, gpointer data) {
	gps_main_window_destroy(w, data);
}

static void
gps_main_window_button_about_clicked(GtkWidget* w, gpointer data) {
	gps_about_show();
}

static GtkTextBuffer* 
create_text_buffer(void) {
	GtkTextBuffer *buffer;

	buffer = gtk_text_buffer_new(NULL);
	gtk_text_buffer_create_tag(buffer, "bold",
		"weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag(buffer, "italic",
		"style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_buffer_create_tag(buffer, "red", "foreground", "red", NULL);
	gtk_text_buffer_create_tag(buffer, "blue", "foreground", "blue", NULL);
	gtk_text_buffer_create_tag(buffer, "green", "foreground","green",NULL);

	return (buffer);
}

static void
gps_main_window_destroy(GtkWidget *w, gpointer data) {
	if (_GpsMainWindow) {
		gtk_widget_destroy(_GpsMainWindow);
		_GpsMainWindow = NULL;
	}
	gtk_main_quit();
}

void
gps_main_window_show(GPHost* h) {
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *hbox1;
	GtkWidget *frame;
	GtkWidget *hpaned;
	GtkWidget *sw;
	GtkWidget *label;
	GtkWidget *check;
	GtkWidget *button;
	GtkObject *adj1, *adj2;
	gchar* host = gps_host_get_name(h);
	gint port_start = gps_host_get_port_start(h);
	gint port_end = gps_host_get_port_end(h);

	_GpsMainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(_GpsMainWindow), WINT);
	gtk_window_set_resizable(GTK_WINDOW(_GpsMainWindow), TRUE);
	gtk_widget_set_size_request(_GpsMainWindow, WINW, WINH);
	gtk_container_set_border_width(GTK_CONTAINER(_GpsMainWindow), WINB);
	g_signal_connect(G_OBJECT(_GpsMainWindow), "delete_event",
			 G_CALLBACK(gps_main_window_destroy), NULL);

	hpaned = gtk_hpaned_new();
	gtk_container_add(GTK_CONTAINER(_GpsMainWindow), hpaned);

	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
					    GTK_SHADOW_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	gtk_paned_pack1(GTK_PANED(hpaned), sw, TRUE, TRUE);

	_GpsMainBuffer = create_text_buffer();
	_GpsMainText = gtk_text_view_new_with_buffer(_GpsMainBuffer);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(_GpsMainText), FALSE);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(_GpsMainText), GTK_WRAP_WORD);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(_GpsMainText), FALSE);
	gtk_container_add(GTK_CONTAINER(sw), _GpsMainText);
	gtk_widget_show(_GpsMainText);
	gtk_widget_show(sw);

	vbox = gtk_vbox_new(FALSE, WINB);
	gtk_paned_pack2(GTK_PANED(hpaned), vbox, FALSE, FALSE);

	frame = gtk_frame_new("Datos del servidor");
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	vbox1 = gtk_vbox_new(FALSE, WINB);
	gtk_container_set_border_width(GTK_CONTAINER(vbox1), WINB);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);

	hbox1 = gtk_hbox_new(FALSE, WINB);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	label = gtk_label_new("Direccion:");
	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	entry_server = gtk_entry_new();
	g_signal_connect(G_OBJECT(entry_server), "activate",
			 G_CALLBACK(gps_main_window_button_start_clicked),
			 (gpointer)h);
	gtk_box_pack_end(GTK_BOX(hbox1), entry_server, TRUE, TRUE, 0);
	gtk_widget_show(entry_server);
	gtk_widget_show(hbox1);

	if (host && strlen(host) >= 1)
		gtk_entry_set_text(GTK_ENTRY(entry_server), host);
	else
		gtk_widget_grab_focus(entry_server);

	hbox1 = gtk_hbox_new(FALSE, WINB);
	gtk_box_pack_end(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	label = gtk_label_new("Inicio:");
	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	adj1 = gtk_adjustment_new(DEF_PORT_MIN, PORT_MIN, PORT_MAX, 1, 10, 10);
	spin_sport = gtk_spin_button_new(GTK_ADJUSTMENT(adj1), 1, 0);
	if (port_start)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_sport),
			port_start);
	gtk_box_pack_start(GTK_BOX(hbox1), spin_sport, TRUE, TRUE, 0);
	gtk_widget_show(spin_sport);

	adj2 = gtk_adjustment_new(DEF_PORT_MAX, PORT_MIN, PORT_MAX, 1, 10, 10);
	spin_eport = gtk_spin_button_new(GTK_ADJUSTMENT(adj2), 1, 0);
	if (port_end)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_eport),
			port_end);
	gtk_box_pack_end(GTK_BOX(hbox1), spin_eport, TRUE, TRUE, 0);
	gtk_widget_show(spin_eport);

	label = gtk_label_new("Termino:");
	gtk_box_pack_end(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	gtk_widget_show(hbox1);

	gtk_widget_show(vbox1);
	gtk_widget_show(frame);

	frame = gtk_frame_new("Opciones");
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);

	vbox1 = gtk_vbox_new(FALSE, WINB);
	gtk_container_set_border_width(GTK_CONTAINER(vbox1), WINB);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);

	check=gtk_check_button_new_with_label("Identificar nombre de servicio");
	gtk_box_pack_start(GTK_BOX(vbox1), check, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(check), "toggled",
			 G_CALLBACK(gps_main_window_check_ident_toggled),
			 (gpointer)h);
	gtk_widget_show(check);

	gtk_widget_show(vbox1);
	gtk_widget_show(frame);

	frame = gtk_frame_new("Comandos");
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	gtk_box_pack_end(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	vbox1 = gtk_vbox_new(TRUE, WINB);
	gtk_container_set_border_width(GTK_CONTAINER(vbox1), WINB);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);

	hbox1 = gtk_hbox_new(TRUE, WINB);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	button = gtk_button_new_with_label("Iniciar");
	gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_start_clicked),
			 (gpointer)h);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Detener");
	gtk_box_pack_end(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_stop_clicked),
			 (gpointer)h);
	gtk_widget_show(button);
	gtk_widget_show(hbox1);

	hbox1 = gtk_hbox_new(TRUE, WINB);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	button = gtk_button_new_with_label("Limpiar");
	gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_clear_clicked),
			 NULL);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Guardar");
	gtk_box_pack_end(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_save_clicked),
			 (gpointer)h);
	gtk_widget_show(button);
	gtk_widget_show(hbox1);

	hbox1 = gtk_hbox_new(TRUE, WINB);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	button = gtk_button_new_with_label("Salir");
	gtk_box_pack_start(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_quit_clicked),
			 NULL);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Acerca de");
	gtk_box_pack_end(GTK_BOX(hbox1), button, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(gps_main_window_button_about_clicked),
			 NULL);
	gtk_widget_show(button);
	gtk_widget_show(hbox1);

	gtk_widget_show(vbox1);
	gtk_widget_show(frame);

	gtk_widget_show(vbox);
	gtk_widget_show(hpaned);
	gtk_widget_show(_GpsMainWindow);
}
